@@ -1,12 +1,10 @@
 """toolstr is a suite of str processing tools, including formatting and drawing"""
 
-from .blocks import *
 from .charts import *
-from .formatting import *
-from .indents import *
+from .formats import *
 from .outlines import *
 from .spec import *
 from .tables import *
 
 
-__version__ = '0.4.0'
+__version__ = '0.8.2'
