@@ -4,13 +4,14 @@ import typing
 import types
 from typing_extensions import TypedDict
 
-from .. import formatting
+from .. import formats
 from .. import outlines
 from .. import spec
 from . import multiline_tables
 
 
 if typing.TYPE_CHECKING:
+    from typing_extensions import Literal
     import rich.console
 
     class TableStyleContext(TypedDict):
@@ -50,10 +51,12 @@ def print_table(
     *,
     add_row_index: bool = False,
     row_start_index: int = 1,
+    limit_rows: int | None = None,
+    limit_rows_at: Literal['start', 'middle', 'end'] = 'middle',
     missing_columns: typing.Literal['fill', 'clip', 'error'] = 'error',
     empty_str: str = '',
     format: FormatKwargs | None = None,
-    column_format: ColumnData[FormatKwargs] | None = None,
+    column_formats: ColumnData[FormatKwargs] | None = None,
     return_str: bool = False,
     #
     # io
@@ -65,6 +68,7 @@ def print_table(
     label_location: HeaderLocation | None = None,
     max_table_width: int | None = None,
     column_widths: typing.Sequence[int] | None = None,
+    max_column_widths: ColumnData[int] | None = None,
     indent: str | int | None = None,
     outer_gap: int | str | None = None,
     column_gap: int | str | None = None,
@@ -81,7 +85,7 @@ def print_table(
     label_vertical_justify: ColumnData[spec.VerticalJustification]
     | None = 'bottom',
     style: Style | None = None,
-    column_style: ColumnData[Style] | None = None,
+    column_styles: ColumnData[Style] | None = None,
     label_style: ColumnData[Style] | None = None,
 ) -> str | None:
 
@@ -93,21 +97,24 @@ def print_table(
 
     # add row index
     rows, labels = _add_index(rows, labels, add_row_index, row_start_index)
+    if limit_rows is not None and len(rows) > limit_rows:
+        rows = clip_rows(rows, n=limit_rows, clip_position=limit_rows_at)
 
     # convert cells and labels to str
     str_cells, str_labels, column_widths, use_styles = _stringify_all(
         rows=rows,
         labels=labels,
         column_widths=column_widths,
+        max_column_widths=max_column_widths,
         format=format,
-        column_format=column_format,
+        column_formats=column_formats,
         empty_str=empty_str,
         justify=justify,
         column_justify=column_justify,
         label_justify=label_justify,
         label_vertical_justify=label_vertical_justify,
         style=style,
-        column_style=column_style,
+        column_styles=column_styles,
         label_style=label_style,
         use_styles=use_styles,
         add_row_index=add_row_index,
@@ -193,31 +200,34 @@ def _fix_missing_data(
     missing_columns: typing.Literal['clip', 'fill', 'error'],
     empty_str: str,
 ) -> tuple[list[typing.Sequence[typing.Any]], typing.Sequence[str] | None]:
-    min_columns = 1_000_000_000
-    max_columns = 0
-    for row in rows:
-        n_row_columns = len(row)
-        min_columns = min(min_columns, n_row_columns)
-        max_columns = max(max_columns, n_row_columns)
-    if labels is not None:
-        min_columns = min(min_columns, len(labels))
-        max_columns = max(max_columns, len(labels))
-    if min_columns != max_columns:
-        if missing_columns == 'error':
-            raise Exception(
-                'different numbers of columns, use missing_columns="clip" or missing_columns="fill"'
-            )
-        elif missing_columns == 'clip':
-            rows = [row[:min_columns] for row in rows]
-            if labels is not None:
-                labels = labels[:min_columns]
-        elif missing_columns == 'fill':
-            rows = [
-                list(row) + [empty_str] * (max_columns - len(row))
-                for row in rows
-            ]
-            if labels is not None:
-                labels = list(labels) + [''] * (max_columns - len(labels))
+
+    if len(rows) > 0:
+        min_columns = 1_000_000_000
+        max_columns = 0
+        for row in rows:
+            n_row_columns = len(row)
+            min_columns = min(min_columns, n_row_columns)
+            max_columns = max(max_columns, n_row_columns)
+        if labels is not None:
+            min_columns = min(min_columns, len(labels))
+            max_columns = max(max_columns, len(labels))
+        if min_columns != max_columns:
+            if missing_columns == 'error':
+                raise Exception(
+                    'different numbers of columns, use missing_columns="clip" or missing_columns="fill"'
+                )
+            elif missing_columns == 'clip':
+                rows = [row[:min_columns] for row in rows]
+                if labels is not None:
+                    labels = labels[:min_columns]
+            elif missing_columns == 'fill':
+                rows = [
+                    list(row) + [empty_str] * (max_columns - len(row))
+                    for row in rows
+                ]
+                if labels is not None:
+                    labels = list(labels) + [''] * (max_columns - len(labels))
+
     return rows, labels
 
 
@@ -245,9 +255,10 @@ def _stringify_all(
     rows: typing.Sequence[typing.Sequence[typing.Any]],
     labels: typing.Sequence[str] | None,
     column_widths: typing.Sequence[int] | None,
+    max_column_widths: ColumnData[int] | None,
     empty_str: str,
     format: FormatKwargs | None,
-    column_format: ColumnData[FormatKwargs] | None,
+    column_formats: ColumnData[FormatKwargs] | None,
     add_row_index: bool,
     justify: spec.HorizontalJustification,
     column_justify: ColumnData[spec.HorizontalJustification] | None,
@@ -257,7 +268,7 @@ def _stringify_all(
     label_vertical_justify: ColumnData[spec.VerticalJustification] | None,
     use_styles: bool | None,
     style: Style | None,
-    column_style: ColumnData[Style] | None,
+    column_styles: ColumnData[Style] | None,
     label_style: ColumnData[Style] | None,
 ) -> tuple[list[list[str]], list[list[str]], typing.Sequence[int], bool]:
 
@@ -270,11 +281,11 @@ def _stringify_all(
         return [], [], [], False
 
     # convert cells to str
-    column_format = _convert_column_dict_to_list(
-        column_format, n_columns, labels
+    column_formats = _convert_column_dict_to_list(
+        column_formats, n_columns, labels
     )
     str_cells = [
-        _stringify_cells(row, format, column_format, empty_str) for row in rows
+        _stringify_cells(row, format, column_formats, empty_str) for row in rows
     ]
     if labels is not None:
         label_lines = multiline_tables._split_multiline_row(
@@ -290,9 +301,18 @@ def _stringify_all(
 
     # determine column widths
     if column_widths is None:
-        column_widths = _get_column_widths(str_cells + str_labels)
+        if isinstance(max_column_widths, list) and add_row_index:
+            max_column_widths = [max_column_widths[0]] + max_column_widths
+        max_column_widths = _convert_column_dict_to_list(
+            max_column_widths, n_columns, labels
+        )
+        column_widths = _get_column_widths(
+            str_cells + str_labels, max_column_widths
+        )
 
     # trim and justify cells to column widths
+    if isinstance(column_justify, list) and add_row_index:
+        column_justify = [column_justify[0]] + column_justify
     column_justify = _convert_column_dict_to_list(
         column_justify, n_columns, labels
     )
@@ -318,8 +338,8 @@ def _stringify_all(
     if use_styles:
 
         # arrange column style specifications
-        column_style = _convert_column_dict_to_list(
-            column_style, n_columns, labels
+        column_styles = _convert_column_dict_to_list(
+            column_styles, n_columns, labels
         )
 
         # style rows
@@ -327,7 +347,7 @@ def _stringify_all(
             rows=rows,
             str_rows=str_cells,
             style=style,
-            column_style=column_style,
+            column_styles=column_styles,
             labels=labels,
             str_labels=str_labels,
         )
@@ -347,7 +367,7 @@ def _stringify_all(
             rows=str_labels,
             str_rows=str_labels,
             style=None,
-            column_style=label_style,
+            column_styles=label_style,
             labels=labels,
             str_labels=str_labels,
         )
@@ -355,24 +375,37 @@ def _stringify_all(
     return str_cells, str_labels, column_widths, use_styles
 
 
-def _get_column_widths(str_cells: list[list[str]]) -> list[int]:
+def _get_column_widths(
+    str_cells: list[list[str]],
+    max_column_widths: typing.Sequence[int | None] | None,
+) -> list[int]:
+
     if len(str_cells) > 0:
         n_columns = len(str_cells[0])
     else:
         return []
-    max_column_widths: list[int] = [0] * n_columns
+    column_widths: list[int] = [0] * n_columns
     for row_str_cells in str_cells:
         for c, str_cell in enumerate(row_str_cells):
+
             if '[' in str_cell:
                 import rich.text
 
                 cell_width = rich.text.Text.from_markup(str_cell).cell_len
             else:
                 cell_width = len(str_cell)
-            if cell_width > max_column_widths[c]:
-                max_column_widths[c] = cell_width
+            if cell_width > column_widths[c]:
+                column_widths[c] = cell_width
 
-    return max_column_widths
+    if max_column_widths is not None:
+        for c, max_column_width in enumerate(max_column_widths):
+            if (
+                max_column_width is not None
+                and column_widths[c] > max_column_width
+            ):
+                column_widths[c] = max_column_width
+
+    return column_widths
 
 
 @typing.overload
@@ -432,7 +465,7 @@ def _convert_column_dict_to_list(
 def _stringify_cells(
     row: typing.Sequence[typing.Any],
     format: FormatKwargs | None,
-    column_format: typing.Sequence[None | typing.Mapping[str, typing.Any]]
+    column_formats: typing.Sequence[None | typing.Mapping[str, typing.Any]]
     | None,
     empty_str: str,
 ) -> list[str]:
@@ -451,16 +484,16 @@ def _stringify_cells(
                 # get format kwargs
                 cell_format: typing.Mapping[str, typing.Any] | None
                 cell_format = None
-                if column_format is not None:
-                    cell_format = column_format[c]
+                if column_formats is not None:
+                    cell_format = column_formats[c]
                 if format is not None and cell_format is None:
                     cell_format = format
 
                 # format as str
                 if cell_format is not None:
-                    as_str = formatting.format(cell, **cell_format)
+                    as_str = formats.format(cell, **cell_format)
                 else:
-                    as_str = formatting.format(cell)
+                    as_str = formats.format(cell)
 
             else:
                 as_str = str(cell)
@@ -482,17 +515,28 @@ def _trim_justify(
 ) -> list[str]:
     """trim or justify cells in row to target sizes"""
 
+    import rich.text
+
     if column_justify is not None and len(column_justify) != len(str_row):
         raise Exception('wrong length of list')
 
     output = []
     for c, cell in enumerate(str_row):
-        length = len(cell)
+        rich_cell = rich.text.Text.from_markup(cell)
+        length = len(rich_cell.plain)
+
         width = column_widths[c]
-        if length >= width:
+        if length == width:
+            output.append(cell)
+        elif length > width:
 
             # trim
-            output.append(cell[:width])
+
+            if width >= 3:
+                trimmed = rich_cell.fit(width - 3)[0].markup + '...'
+            else:
+                trimmed = '.' * width
+            output.append(trimmed)
 
         else:
 
@@ -504,7 +548,7 @@ def _trim_justify(
                 cell_justify = justify
 
             # justify
-            output.append(formatting.hjustify(cell, cell_justify, width))
+            output.append(formats.hjustify(cell, cell_justify, width))
 
     return output
 
@@ -515,7 +559,7 @@ def _stylize_rows(
     labels: typing.Sequence[typing.Any] | None,
     str_labels: list[list[str]],
     style: Style | None,
-    column_style: ColumnData[Style] | None,
+    column_styles: ColumnData[Style] | None,
 ) -> list[list[str]]:
 
     stylized_rows = []
@@ -525,8 +569,8 @@ def _stylize_rows(
         for c, (cell, str_cell) in enumerate(zip(row, str_row)):
 
             # use column style if specified, otherwise use global style
-            if column_style is not None and column_style[c] is not None:
-                cell_style: Style | None = column_style[c]
+            if column_styles is not None and column_styles[c] is not None:
+                cell_style: Style | None = column_styles[c]
             elif style is not None:
                 cell_style = style
             else:
@@ -550,9 +594,7 @@ def _stylize_rows(
             if cell_style is not None:
                 if not isinstance(cell_style, str):
                     raise Exception('could not convert style to str')
-                str_cell = (
-                    '[' + cell_style + ']' + str_cell + '[/' + cell_style + ']'
-                )
+                str_cell = formats.add_style(str_cell, cell_style)
             stylized_row.append(str_cell)
 
         stylized_rows.append(stylized_row)
@@ -595,15 +637,23 @@ def _convert_table_to_str(
     separator_indices: set[int],
 ) -> str:
 
+    import rich.text
+
     # use compact format
     if compact:
         if outer_gap is None:
             outer_gap = ''
         if column_gap is None:
             column_gap = ''
-        border = outlines.get_border_chars()
+
+        if isinstance(border, str):
+            border = outlines.get_border_chars_by_name(border)
+        elif border is None:
+            border = outlines.get_border_chars()
+        else:
+            raise Exception('bad format for compact border: ' + str(border))
         border['vertical'] = ' ' * int(compact)
-        border['cross'] = border['horizontal']
+        border['cross'] = border['horizontal'] * int(compact)
 
     # determine border styles
     if border is None:
@@ -836,11 +886,14 @@ def _convert_table_to_str(
         ellipses = True
         trimmed_lines = []
         for line in lines:
-            if len(line) > max_table_width:
+            rich_line = rich.text.Text.from_markup(line)
+            line_len = rich_line.cell_len
+            if line_len > max_table_width:
                 if ellipses:
-                    line = line[: max_table_width - 3] + '...'
+                    line = rich_line.fit(max_table_width - 3)[0].markup + '...'
                 else:
-                    line = line[:max_table_width]
+                    line = rich_line.fit(max_table_width)[0].markup
+
             trimmed_lines.append(line)
         lines = trimmed_lines
 
@@ -891,3 +944,35 @@ def _print_table(
 
     else:
         print(table_as_str)
+
+
+def clip_rows(
+    rows: list[typing.Sequence[typing.Any]],
+    n: int,
+    *,
+    clip_position: Literal['start', 'middle', 'end'] = 'end',
+    fill: str | None = '...',
+) -> list[typing.Sequence[typing.Any]]:
+
+    if len(rows) <= n:
+        return rows
+
+    # construct fill
+    if fill is not None:
+        n_columns = len(rows[0])
+        fill_rows: list[typing.Sequence[typing.Any]] = [[fill] * n_columns]
+        n = n - 1
+
+    # clip
+    if clip_position == 'start':
+        return fill_rows + rows[-n:]
+    elif clip_position == 'end':
+        return rows[:n] + fill_rows
+    elif clip_position == 'middle':
+        import math
+
+        n_head = math.ceil(n / 2)
+        n_tail = n - n_head
+        return rows[:n_head] + fill_rows + rows[-n_tail:]
+    else:
+        raise Exception('invalid clip_rows specification')
